# Emmy version 1.0.0
# EmmyDDCache.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Unary operations

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDDNode import *
from EmmyDDCacheRecord import *

# 2) Creators
# -----------

class EmmyDDCache:
  def __init__(self, size):
    self._size = size
    self._records = [EmmyDDCacheRecord() for i in range(0, size)]

  def Reset(self):
    for record in self._records:
      record.Reset()

  def RemoveOperationsOfCode(self, operationCode):
    for record in self._records:
      if record.GetOperationCode()==operationCode:
        record.Reset()

# 3) Unary operations
# -------------------

  def LookUpUnaryOperation(self, operationCode, argument):
    hashCode = self.ComputeHashCodeUnaryOperation(operationCode, argument)
    record = self._records[hashCode]
    return record.MatchUnary(operationCode, argument)

  def InsertUnaryOperation(self, operationCode, argument, result):
    hashCode = self.ComputeHashCodeUnaryOperation(operationCode, argument)
    self._records[hashCode].SetUnary(operationCode, argument, result)

  def ComputeHashCodeUnaryOperation(self, operationCode, argument):
    return (7*operationCode + 13*argument.GetNumber()) % self._size

# 4) Binary operations
# --------------------

  def LookUpBinaryOperation(self, operationCode, argument1, argument2):
    hashCode = self.ComputeHashCodeBinaryOperation(operationCode, argument1, argument2)
    record = self._records[hashCode]
    return record.MatchBinary(operationCode, argument1, argument2)

  def InsertBinaryOperation(self, operationCode, argument1, argument2, result):
    hashCode = self.ComputeHashCodeBinaryOperation(operationCode, argument1, argument2)
    self._records[hashCode].SetBinary(operationCode, argument1, argument2, result)

  def ComputeHashCodeBinaryOperation(self, operationCode, argument1, argument2):
    return (7*operationCode + 13*argument1.GetNumber() + 17*argument2.GetNumber()) % self._size

# 4) Ternary operations
# ---------------------

  def LookUpTernaryOperation(self, operationCode, argument1, argument2, argument3):
    hashCode = self.ComputeHashCodeTernaryOperation(operationCode, argument1, argument2, argument3)
    record = self._records[hashCode]
    return record.MatchTernary(operationCode, argument1, argument2, argument3)

  def InsertTernaryOperation(self, operationCode, argument1, argument2, argument3, result):
    hashCode = self.ComputeHashCodeTernaryOperation(operationCode, argument1, argument2, argument3)
    self._records[hashCode].SetTernary(operationCode, argument1, argument2, argument3, result)

  def ComputeHashCodeTernaryOperation(self, operationCode, argument1, argument2, argument3):
    return (7*operationCode + 13*argument1.GetNumber() + 17*argument2.GetNumber() + 23*argument3.GetNumber()) % self._size
