# Emmy version 1.0.0
# EmmyDDCacheRecord.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDDNode import *

# 2) Creators
# -----------

class EmmyDDCacheRecord:
  NO_OPERATION = 0

  def __init__(self):
    self._operationCode = EmmyDDCacheRecord.NO_OPERATION
    self._arguments = []
    self._result = None

# 3) Access methods
# -----------------

  def GetArity(self):
    return len(self._arguments)

  def GetOperationCode(self):
    return self._operationCode

  def GetArgument(self, index):
    if index<0 or index>=len(self._arguments):
      return None
    return self._arguments[index]

  def GetResult(self):
    return self._result

  def Reset(self):
    self._operationCode = EmmyDDCacheRecord.NO_OPERATION
    self._arguments = []
    self._result = None

  def MatchUnary(self, operationCode, argument1):
    if self.GetArity()==1 and self._operationCode==operationCode and self._arguments[0]==argument1:
      return self._result
    return None

  def SetUnary(self, operationCode, argument1, result):
    self._operationCode = operationCode
    self._arguments = [argument1]
    self._result = result
    return self

  def MatchBinary(self, operationCode, argument1, argument2):
    if self.GetArity()==2 and self._operationCode==operationCode and self._arguments[0]==argument1 and self._arguments[1]==argument2:
      return self._result
    return None

  def SetBinary(self, operationCode, argument1, argument2, result):
    self._operationCode = operationCode
    self._arguments = [argument1, argument2]
    self._result = result
    return self

  def MatchTernary(self, operationCode, argument1, argument2, argument3):
    if self.GetArity()==2 and self._operationCode==operationCode and self._arguments[0]==argument1 and self._arguments[1]==argument2 and self._arguments[2]==argument3:
      return self._result
    return None

  def SetTernary(self, operationCode, argument1, argument2, argument3, result):
    self._operationCode = operationCode
    self._arguments = [argument1, argument2, argument3]
    self._result = result
    return self

