# Emmy version 1.0.0
# EmmyFDSDomain.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods
# 4) Constants

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDecisionDiagrams import *

from EmmyFDSConstant import *

# 2) Creators
# -----------

class EmmyFDSDomain:
  def __init__(self, identifier, domain):
    self._identifier = identifier
    self._domain = domain
    self._fdsConstants = []
    for constant in domain.GetConstants():
      fdsConstant = EmmyFDSConstant(constant)
      self._fdsConstants.append(fdsConstant)
    
# 3) Access methods
# -----------------

  def GetIdentifier(self):
    return self._identifier

  def GetDomain(self):
    return self._domain

# 4) Constants
# ------------

  def GetSize(self):
    return len(self._fdsConstants)

  def GetFDSConstants(self):
    return self._fdsConstants

  def GetFDSConstant(self, constant):
    for fdsConstant in self._fdsConstants:
      if S2MLConstant.IsEq(constant, fdsConstant.GetConstant()):
        return fdsConstant
    return None

  def SortFDSConstants(self):
    fdsCandidates = self._fdsConstants
    self._fdsConstants = []
    for fdsCandidate in fdsCandidates:
      index = 0
      for fdsConstant in self._fdsConstants:
        if fdsCandidate.IsFDSBigger(fdsConstant):
          break
        index += 1
      self._fdsConstants.insert(index, fdsCandidate)