# Emmy version 1.0.0
# EmmyFDSVariable.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods
# 4) Literals
# 5) Handles

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDecisionDiagrams import *

from EmmyFDSConstant import *
from EmmyFDSDomain import *
from EmmyFDSLiteral import *

# 2) Creators
# -----------

class EmmyFDSVariable:
  def __init__(self, identifier):
    self._identifier = identifier
    self._variable = None
    self._observer = None
    self_fdsDomain = None
    from EmmyFDSLiteral import EmmyFDSLiteral
    self._fdsLiterals = []
    self._ddHandles = dict()
    self._decisionDiagram = None

# 3) Access methods
# -----------------

  def GetVariable(self):
    return self._variable

  def SetVariable(self, variable):
    self._variable = variable

  def GetObserver(self):
    return self._observer

  def SetObserver(self, observer):
    self._observer = observer

  def GetFDSDomain(self):
    return self._fdsDomain

  def SetFDSDomain(self, fdsDomain):
    self._fdsDomain = fdsDomain

# 4) Literals
# -----------

  def GetNumberOfFDSLiterals(self):
    return len(self._fdsLiterals)

  def GetFDSLiterals(self):
    return self._fdsLiterals

  def GetFDSLiteral(self, index):
    if index<0 or index>=len(self._fdsLiterals):
      return None
    return self._fdsLiterals[index]
 
  def LookForFDSLiteral(self, fdsConstant):
    for fdsLiteral in self._fdsLiterals:
      if fdsLiteral.GetFDSConstant()==fdsConstant:
        return fdsLiteral
    return None

  def NewFDSLiteral(self, fdsConstant):
    fdsLiteral = self.LookForFDSLiteral(fdsConstant)
    if fdsLiteral==None:
      fdsLiteral = EmmyFDSLiteral(self, fdsConstant)
      self._fdsLiterals.append(fdsLiteral)
    return fdsLiteral

# 5) Handles
# ----------

  def ResetDDHandles(self):
    self._ddHandles.clear()

  def GetDDHandles(self):
    return self._ddHandles

  def LookForDDHandle(self, name):
    return self._ddHandles.get(name, None)

  def NewDDHandle(self, name, rootNode):
    ddHandle = self.LookForDDHandle(name)
    if ddHandle==None:
      ddHandle = EmmyDDHandle(name, rootNode)
      self._ddHandles[name] = ddHandle
    else:
      ddHandle.SetRootNode(rootNode)
    return ddHandle

  def GetDecisionDiagram(self):
    return self._decisionDiagram

  def SetDecisionDiagram(self, decisionDiagram):
    self._decisionDiagram = decisionDiagram
