# Emmy version 1.0.0
# EmmyEngine.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Settings
# 4) Algorithms

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDecisionDiagrams import *
from EmmyFiniteDegradationStructures import *

from EmmyModels import *
from EmmyInputsOutputs import *

from EmmyFlattener import *
from EmmyChecker import *
from EmmyRewriter import *
from EmmyInterpreter import *
from EmmyDecisionDiagramBuilder import *
from EmmyStochasticSimulator import *

# 2) Creator
# ----------

class EmmyEngine(S2MLEngine):

  def __init__(self):
    S2MLEngine.__init__(self)
    self._decisionDiagramBuilder = None
    self._stochasticSimulator = None
    self._interpreter = None
    self._sourceModel = EmmyModel()
    self._fdsModel = EmmyFDSModel()
    self._ddManager = EmmyDDManager(1024, 1024)

# 3) Settings
# -----------

  def CreateTools(self):
    self._lexer = EmmyLexer(self)
    self._parser = EmmyParser(self)
    self._expressionManager = S2MLStochasticExpressionManager(self)
    self._printer = EmmyPrinter(self)
    self._flattener = EmmyFlattener(self)
    self._checker = EmmyChecker(self)
    self._rewriter = EmmyRewriter(self)
    self._decisionDiagramBuilder = EmmyDecisionDiagramBuilder(self)
    self._stochasticSimulator = EmmyStochasticSimulator(self)
    self._interpreter = EmmyInterpreter(self)

  def GetDecisionDiagramBuilder(self):
    return self._decisionDiagramBuilder

  def GetStochasticSimulator(self):
    return self._stochasticSimulator

  def GetFDSModel(self):
    return self._fdsModel

  def GetDDManager(self):
    return self._ddManager

# 4) Methods to import, instantiate and export models
# ---------------------------------------------------

  def NewModel(self):
    return EmmyModel()

  def ResetModel(self):
    self._sourceModel = EmmyModel()
    self._targetModel = None
    self._targetBlock = None
    return 0
