# S2MLCore version 1.0.0
# S2MLReal.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Constants
# 3) Assignments and conversions
# 4) Boolean operations
# 5) Arithmetic operations
# 6) String operations

# 1) Imported modules
# -------------------

import string
import sys
import re
import math
import mpmath

# 2) Creators
# -----------

class S2MLReal:
  _Manager = None

  def __init__(self):
    mpmath.mp.dps = 30
    self._epsilon = mpmath.mpf(1.0e-20)

  @staticmethod
  def GetManager():
    if S2MLReal._Manager==None:
      S2MLReal._Manager = S2MLReal()
    return S2MLReal._Manager

  @staticmethod
  def New(value):
    real = mpmath.mpf(value)
    manager = S2MLReal.GetManager()
    if mpmath.fabs(real)<manager._epsilon:
      return mpmath.mpf(0.0)
    return real

  @staticmethod
  def PosInf():
    return math.inf

  @staticmethod
  def NegInf():
    return -math.inf

  @staticmethod
  def Eq(value1, value2):
    return value1==value2

  @staticmethod
  def AlmostEq(value1, value2, epsilon):
    return mpmath.almosteq(value1, value2, epsilon)

  @staticmethod
  def Df(value1, value2):
    return value1!=value2

  @staticmethod
  def Lt(value1, value2):
    return value1<value2

  @staticmethod
  def Gt(value1, value2):
    return value1>value2

  @staticmethod
  def Leq(value1, value2):
    return value1<=value2

  @staticmethod
  def Geq(value1, value2):
    return value1>=value2

  @staticmethod
  def Add(value1, value2):
    manager = S2MLReal.GetManager()
    value = mpmath.fadd(value1, value2)
    if mpmath.fabs(value)<manager._epsilon:
      return mpmath.mpf(0.0)
    return mpmath.mpf(value)

  @staticmethod
  def Sub(value1, value2):
    manager = S2MLReal.GetManager()
    value = mpmath.fsub(value1, value2)
    if mpmath.fabs(value)<manager._epsilon:
      return mpmath.mpf(0.0)
    return mpmath.mpf(value)

  @staticmethod
  def Mul(value1, value2):
    manager = S2MLReal.GetManager()
    value = mpmath.fmul(value1, value2)
    if mpmath.fabs(value)<manager._epsilon:
      return mpmath.mpf(0.0)
    return mpmath.mpf(value)

  @staticmethod
  def Div(value1, value2):
    manager = S2MLReal.GetManager()
    value = mpmath.fdiv(value1, value2)
    if mpmath.fabs(value)<manager._epsilon:
      return mpmath.mpf(0.0)
    return mpmath.mpf(value)

  @staticmethod
  def Neg(value1):
    manager = S2MLReal.GetManager()
    value = mpmath.fneg(value1)
    if mpmath.fabs(value)<manager._epsilon:
      return mpmath.mpf(0.0)
    return mpmath.mpf(value)

  @staticmethod
  def Abs(value1):
    manager = S2MLReal.GetManager()
    value = mpmath.fabs(value1)
    if value<manager._epsilon:
      return mpmath.mpf(0.0)
    return mpmath.mpf(value)

  @staticmethod
  def Ceil(value1):
    manager = S2MLReal.GetManager()
    value = mpmath.ceil(value1)
    if mpmath.fabs(value)<manager._epsilon:
      return mpmath.mpf(0.0)
    return mpmath.mpf(value)

  @staticmethod
  def ToString(value1):
    return mpmath.nstr(value1)

